
import os
import base64
import inspect
import logging

import pyaas

def generateCookieSecret(path):
    secret = base64.b64encode(os.urandom(32))
    with open(path, 'w') as fp:
        fp.write(secret)
    return secret

# TODO: add functionality to manage basic authentication

def setPrefix(prefix=None):
    if not prefix:
        # inspect who called this function
        frames = inspect.getouterframes(inspect.currentframe())
        # get the caller frame
        frame = frames[1]
        # get the filename of the caller
        prefix = os.path.abspath(frame[1])
        # get the directory name of the file
        prefix = os.path.dirname(prefix)

        if prefix.endswith(os.path.sep + 'bin'):
            prefix = os.path.join(prefix, '..')
            prefix = os.path.abspath(prefix)

    pyaas.prefix = os.path.abspath(prefix)
    logging.debug('Setting prefix to "%s"', pyaas.prefix)

def setNameSpace(namespace=None):
    if namespace is None:
        # inspect who called this function
        frames = inspect.getouterframes(inspect.currentframe())
        # get the caller frame
        frame = frames[1]
        namespace = frame[0].f_code.co_name
        logging.debug('Setting namespace to "%s"', namespace)

    pyaas.namespace = namespace




import tornado.web

class Base(tornado.web.RequestHandler):
    def get_current_user(self):
        uid  = self.get_secure_cookie('uid')
        if uid is None:
            self.clear_cookie('uid')
        return uid

class Index(Base):
    def initialize(self, template='index'):
        self.template = template + '.html'

    def get(self, template=None):
        template = template + '.html' if template else self.template
        self.render(template)

class Protected(Base):
    def initialize(self, template='index'):
        self.template = template + '.html'

    @tornado.web.authenticated
    def get(self, template=None):
        template = template + '.html' if template else self.template
        self.render(template)




import sys
import os

VERSION = '0.2.8'

# a generic error class for throwing exceptions
class error(Exception):
    def __init__(self, fmt, *args):
        self.str = fmt % args

    def __str__(self):
        return self.str


from . import settings
from . import util
from . import server

util.setPrefix()
util.setNameSpace('')



import sys
import logging

import pymongo
import gridfs
import bson
import bson.json_util

import pyaas

def connect():
    dbserver = pyaas.args.dbhost  or pyaas.config.get('mongo', 'server')
    dbname   = pyaas.args.dbname  or pyaas.config.get('mongo', 'database')
    store    = pyaas.args.dbstore or pyaas.config.get('mongo', 'store')

    try:
        pyaas.mongo = pymongo.Connection(dbserver)
    except pymongo.errors.AutoReconnect:
        raise pyaas.error('Could not connect to database')

    if pyaas.args.reset and pyaas.args.confirm:
        pyaas.mongo.drop_database(dbname)
        pyaas.mongo.drop_database(store)
        sys.exit(0)
    elif pyaas.args.reset:
        logging.error('Reset issued without confirmation')
        sys.exit(-1)

    pyaas.mongo.document_class = bson.SON

    pyaas.db = pyaas.mongo[dbname]
    pyaas.fs = gridfs.GridFS(pyaas.mongo[store])




import os
import socket
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import pyaas

import tornado.web
import tornado.ioloop

pyaas.ioloop = tornado.ioloop.IOLoop.instance()

from . import handlers

class Application(tornado.web.Application):
    def __init__(self, section='server'):
        # get the interface and port to listen on
        self.addr = pyaas.args.address or pyaas.config.get(section, 'address')
        self.port = pyaas.args.port or pyaas.config.getint(section, 'port')

        # load the cookie secret used to encrypt cookies
        cookie_path = os.path.join(pyaas.prefix, 'etc', pyaas.namespace, 'cookie.secret')

        if pyaas.args.newcookie:
            cookie_secret = pyaas.util.generateCookieSecret(cookie_path)
        else:
            try:
                with open(cookie_path, 'rb') as fp:
                    cookie_secret = fp.read(44)
            except IOError:
                cookie_secret = pyaas.util.generateCookieSecret(cookie_path)

        # URI patterns
        if not hasattr(self, 'ssl_options'):
            self.ssl_options = None

        # URI patterns
        if not hasattr(self, 'patterns'):
            self.patterns = []

        # Tornado settings
        self.settings = dict(
            static_path   = os.path.join(pyaas.prefix, 'share', pyaas.namespace, 'static'),
            template_path = os.path.join(pyaas.prefix, 'share', pyaas.namespace, 'templates'),
            cookie_secret = cookie_secret,
            debug         = False
            )

        # useful during development
        if pyaas.args.debug:
            self.settings['debug'] = True

        # get the auth mechanism if any
        try:
            auth = pyaas.config.get(section, 'auth')
        except:
            auth = None

        if auth:
            logging.debug('Enabling authentication: %s', auth)

            # all auth mechanisms need to be in this path
            path = 'pyaas.handlers.auth.' + auth
            try:
                module = __import__(path)
            except ImportError:
                raise pyaas.error('Unknown auth method: %s', auth)

            # drill done to the actual module
            for name in path.split('.')[1:]:
                module = getattr(module, name)

            try:
                Initialize = getattr(module, 'Initialize')
                logging.debug('Initializing authentication: %s', auth)
                Initialize(**dict(pyaas.config.items(auth, True)))
            except AttributeError:
                # no intialization needed
                pass
            except configparser.NoSectionError:
                raise pyaas.error('Missing configurationf for %s', auth)

            # get the Login class from the module
            try:
                Login = getattr(module, 'Login')
            except AttributeError:
                raise pyaas.error('Auth module is missing Login class')

            # extend the patterns and settings accordingly
            self.patterns.extend([
                ( r'/login',  Login                      ),
                ( r'/logout', pyaas.handlers.auth.Logout ),
                ])

            self.settings['login_url'] = '/login'

    def Listen(self):
        # initialize here so patterns and settings can be extended by plugins
        tornado.web.Application.__init__(self, self.patterns, **self.settings)

        try:
            self.listen(self.port, self.addr, ssl_options=self.ssl_options)
        except socket.gaierror as e:
            if 8 == e.errno:
                raise pyaas.error('Invalid address specified "%s"' % self.addr)
            raise pyaas.error('Could not listen: %s' % e)
        except socket.error as e:
            raise pyaas.error('Could not listen: %s' % e)

        logging.info('Listening on %s:%d', self.addr, self.port)

        pyaas.ioloop.start()

    def Stop(self):
        pyaas.ioloop.stop()





import os

import pyaas

try:
    import psycopg2
    import psycopg2.extras
except ImportError:
    raise pyaas.error('Missing pgsql module')

class Database:
    def __init__(self, **kwds):
        try:
            self.conn = psycopg2.connect(**kwds)
        except psycopg2.OperationalError as e:
            raise pyaas.error('Could not connect to database: %s', e)

        self.cursor = self.conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

    def Initialize(self):
        schema = os.path.join(pyaas.prefix, pyaas.config.get('storage', 'schema'))
        schema = open(schema, 'rb').read()
        self.cursor.execute(schema)
        self.conn.commit()

    def Execute(self, statement, *args):
        try:
            self.cursor.execute(statement, args)
        except Exception as e:
            raise pyaas.error('Exception: %s', e)

        return self.cursor.fetchall()

    def Execute2(self, statement, *args):
        try:
            self.cursor.execute(statement, args)
            self.conn.commit()
        except Exception as e:
            raise pyaas.error('Exception: %s', e)

    def Find(self, table, params=None, sort=None):
        statement = 'SELECT * FROM ' + table;
        if params:
            statement += ' WHERE ' + params
        if sort:
            statement += ''

        try:
            self.cursor.execute(statement)
        except psycopg2.ProgrammingError as e:
            raise pyaas.error('Error in statement: %s', e)

        return self.cursor.fetchall()

    def FindOne(self, table, _id):
        statement = 'SELECT * FROM {0} WHERE id = ?'.format(table)
        self.cursor.execute(statement, [_id])
        return self.cursor.fetchone()

    def Count(self, table):
        statement = 'SELECT count(*) FROM {0}'.format(table)
        self.cursor.execute(statement)
        return self.cursor.fetchone()[0]

    def Insert(self, table, values):
        columns = ','.join('"%s"' % k.lower() for k in values.keys())
        placeholder = ','.join('%s' for x in xrange(len(values)))
        statement = 'INSERT INTO {0} ({1}) VALUES ({2})'.format(table, columns, placeholder)

        try:
            self.cursor.execute(statement, values.values())
        except psycopg2.IntegrityError:
            self.conn.rollback()
        except psycopg2.ProgrammingError as e:
            raise pyaas.error('Postgres Programming Error: %s', e)
        else:
            self.conn.commit()

    def Update(self, table, values):
        _id = values['id']
        columns = ','.join(s + '=?' for s in values.keys())
        statement = 'UPDATE {0} SET {1} WHERE id=?'.format(table, columns, _id)
        try:
            self.cursor.execute(statement, values.values() + [_id])
        except psycopg2.ProgrammingError:
            raise

        self.conn.commit()

    def Remove(self, table, _id):
        statement = 'DELETE FROM {0} WHERE id = ?'.format(table)
        self.cursor.execute(statement, [_id])
        self.conn.commit()



#!/usr/bin/env python

import os
import zipfile

from distutils.core import setup

zf = zipfile.ZipFile(os.path.join('pyaas', 'skel.zip'), 'w')
for base,directories,filenames in os.walk('skel'):
    for filename in filenames:
        path = os.path.join(base, filename)
        zf.write(path, path[5:])
zf.close()

setup(
    name = 'pyaas',
    version = '0.2.9',
    author = 'Matthew Oertle',
    author_email = 'moertle@gmail.com',
    url = 'https://github.com/moertle/pyaas',
    license  = 'MIT',
    description = 'Python-as-a-Service is a set of utilities for quickly creating Tornado applications.',
    long_description = open('README.rst').read(),
    packages = [
        'pyaas',
        'pyaas.handlers',
        'pyaas.handlers.auth',
        'pyaas.handlers.ws',
        'pyaas.storage',
        'pyaas.storage.engines',
        ],
    package_data = {
        'pyaas': ['skel.zip']
        },
    install_requires = [
        "tornado >= 3.0",
        ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Internet :: WWW/HTTP",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        ],
    zip_safe = False
)



#!/usr/bin/env python

# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# -----------------------------------------------------------------------------

# The above license applies only to the example project. The intent is to allow
# you to freely use this template for your project and license it as you see
# fit. The pyaas library is released under the MIT License.

# -----------------------------------------------------------------------------

# This file can live in the base directory of the project or in a ./bin
# directory and setPrefix (see below) will resolve accordingly

import os
import logging

import pyaas
import pyaas.storage

class Example(pyaas.server.Application):
    def __init__(self):
        # important to use extend if using built-in authentication
        self.patterns = [
            ( r'/',       pyaas.handlers.Index     ),
            ( r'/(main)', pyaas.handlers.Protected ),
            ]

        pyaas.server.Application.__init__(self)

def example():
    # Set the root for pyaas to this directory
    # If a path is not passed then pyaas will derive the path
    pyaas.util.setPrefix()

    # Set the namespace for the project,
    # e.g. etc/example.ini vs etc/example/example.ini
    # -or- share/static/favicon.ico vs share/example/static/favicon.ico
    #pyaas.util.setNameSpace('example')

    # Load settings from the command line and ini files
    pyaas.settings.load()

    # Optionally load any database backend that is configured
    pyaas.storage.initialize()

    app = Example()
    try:
        app.Listen()
    except KeyboardInterrupt:
        app.Stop()

if '__main__' == __name__:
    try:
        example()
    except pyaas.error as e:
        logging.critical('%s', e)
